/****************************************************************
*Copyright (c) 2018 Tecnalia.
*Licensed under MIT License (the "License");
*Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:
*The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.
*
*
*THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*Contributors (in alphabetical order):
*Alberto Molinuevo 								Tecnalia
*Gorka Benguria                                 Tecnalia
*Iñaki Etxaniz                                  Tecnalia
*Juncal Alonso                                  Tecnalia
*Leire Orue-Echevarria                          Tecnalia
*Maria Jose Lopez                               Tecnalia
*Marisa Escalante                               Tecnalia
*
*Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
*****************************************************************************/

package eu.decideh2020.acsmi.monitoring.server.api.impl;

import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.TimerTask;
import java.lang.Object;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
import org.apache.cxf.jaxrs.client.ClientConfiguration;
import org.apache.cxf.jaxrs.client.WebClient;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectWriter;
import com.fasterxml.jackson.jaxrs.json.JacksonJsonProvider;
import com.maxmind.geoip2.DatabaseReader;
import com.maxmind.geoip2.exception.GeoIp2Exception;
import com.maxmind.geoip2.model.CountryResponse;

import eu.DECIDEh2020.appManager.AppManager;
import eu.DECIDEh2020.appManager.exceptions.AppManagerException;
import eu.DECIDEh2020.appManager.exceptions.DECIDEValidationException;
import eu.DECIDEh2020.appManager.models.AppDescription;
import eu.DECIDEh2020.mcsla.core.metrics.MetricsContext;
import eu.DECIDEh2020.mcsla.core.metrics.ServiceObjectiveMetrics;
import eu.decideh2020.ACSmI.discovery.backend.client.api.ServiceResourceApi;
import eu.decideh2020.ACSmI.discovery.backend.client.api.ServiceIncidenceResourceApi;
import eu.decideh2020.ACSmI.discovery.backend.client.model.Service;
import eu.decideh2020.ACSmI.discovery.backend.client.model.ServiceIncidence;
import eu.decideh2020.ACSmI.discovery.backend.client.model.ServiceNFRInfoVO;
import eu.decideh2020.adapt.violationhandler.client.api.ViolationsApi;
import eu.decideh2020.adapt.violationhandler.client.model.Violation;

public class CsSlaAssessment extends TimerTask {
//AppManager appManager;
	//AppDescription appDesription;
	ServiceResourceApi apiacsmi;
	ViolationsApi apivh;
	@Override
	public void run() {
		// TODO Auto-generated method stub
		csnfrassessment();
	}        	

	
	void csnfrassessment () {
     
        for (MonitoredVm vm: monitoringconfiguration.listcsip) {
        	List <Long> csid = new ArrayList();
        	csid.add(Long.parseLong(vm.getCsId()));
        	String locationerror;
        	String availabilityerror;
        	String performanceerror;
        	// to open connection with ACSmI client
    		ServiceIncidenceResourceApi apiacsmi;
			JacksonJsonProvider provider = new JacksonJsonProvider();
			List providers = new ArrayList();
			providers.add(provider);
			//the integration environment where the ACSmI registry is deployed
			apiacsmi = JAXRSClientFactory.create("http://85.91.40.245:8087/acsmiservices", ServiceIncidenceResourceApi.class, providers);
			org.apache.cxf.jaxrs.client.Client clientacsmi = WebClient.client(apiacsmi);
	        ClientConfiguration configacsmi = WebClient.getConfig(clientacsmi); 
	        // end of the instructions to connect}
	        
	     // to open connection with ADAPT VH client
    		JacksonJsonProvider providervh = new JacksonJsonProvider();
    		List providersvh = new ArrayList();
			providersvh.add(providervh);
			//the integration environment where the Violation handler is deployed
			apivh = JAXRSClientFactory.create("http://85.91.40.245:8095/violationhandler", ViolationsApi.class, providersvh);
			org.apache.cxf.jaxrs.client.Client clientvh = WebClient.client(apivh);
	        ClientConfiguration configvh = WebClient.getConfig(clientvh); 
	        // end of the instructions to connect}
	        // Create a variable for each NFR violation 
        	NfrLocation checklocation = new NfrLocation ();
        	locationerror=checklocation.CheckLocation(csid, vm);
        	NfrAvailability checkavailability = new NfrAvailability();
        	availabilityerror= checkavailability.CheckAvailability (vm,vm.getAppdesc());
        	NfrPerformance checkperformance = new NfrPerformance();
        	performanceerror= checkperformance.CheckPerformance (vm,vm.getAppdesc());
        	System.out.println (" **** Returned values \n \n   : " + new Date());
        	System.out.println(locationerror);
        	System.out.println(availabilityerror);
        	System.out.println(performanceerror);
            // IF NFR Location is violated
        	if (locationerror.contains ("Error")) {
        		 // Call Post for creating the incidence for ACSmi registry
        		ServiceIncidence locationincidence = new ServiceIncidence();
    	        locationincidence.setDetail(locationerror);
    	        // tests for the conversion of the timestamp
    	        //SimpleDateFormat sdf = new SimpleDateFormat("YYYY-mm-dd'T'HH:MM:ssZ");
    	       // System.out.println("****** FECHA : " + sdf.format(new Date()));
    	        // locationincidence.setTimestamp (new Date ().getTime());
    	        locationincidence.setTimestamp("2018-11-26T12:15:00.898Z");
    	        locationincidence.setServiceid(Long.parseLong(vm.getCsId()));
    	      	apiacsmi.createServiceIncidenceUsingPOST(locationincidence);
    	      	System.out.println(" ***** Location violation to ACSmI ***" + locationincidence);
    	        // Call VH for creating the violation in the VH
    	        //System.out.println(" ***** Empezamos a generar la violacion al VH  Location ***");
    	        Violation locationviolation = new Violation();
    	        locationviolation.setNfr("Location");
    	        locationviolation.setServiceId(vm.getCsId());
    	        locationviolation.timestamp(new Date ().getTime());
    	        //System.out.println(" ***** terminamos de generar la violacion al VH  Location 1.... ***" + locationviolation);
    	        // Call to ADAPT VH
    	        //apivh.reportViolation(locationviolation);
    	        System.out.println(" ***** Location violation to VH ***" + locationviolation);    	        
    	     }
        	// IF NFR Availability is violated
        	if (availabilityerror.contains ("Error")) {        		
       		    // Call Post for creating the incidence  in ACSmI
        		ServiceIncidence availabilityincidence = new ServiceIncidence();
    	        availabilityincidence.setDetail(availabilityerror);    	       
    	        // tests for the conversion of the timestamp
    	        //SimpleDateFormat sdf = new SimpleDateFormat("YYYY-mm-dd'T'HH:MM:ssZ");
    	       // System.out.println("****** FECHA : " + sdf.format(new Date()));
    	        // locationincidence.setTimestamp (new Date ().getTime());
    	        availabilityincidence.setTimestamp("2018-11-25T12:15:00.898Z");
    	        availabilityincidence.setServiceid(Long.parseLong(vm.getCsId()));
    	        apiacsmi.createServiceIncidenceUsingPOST(availabilityincidence); 
    	        System.out.println(" ***** Availability violation to ACSmI ***" + availabilityincidence);
    	     // Call VH for creating the violation in the VH
    	        Violation availabilityviolation = new Violation();
    	        availabilityviolation.setNfr("Availabilty");
    	        availabilityviolation.setServiceId(vm.getCsId());
    	        availabilityviolation.timestamp(new Date ().getTime());
    	        // Call to ADAPT VH
    	        //apivh.reportViolation(availabilityviolation);
    	        System.out.println(" ***** Availability violation to VH ***" + availabilityviolation);
    	        }
        	// IF NFR performance is violated
        	if (performanceerror.contains ("Error")) { 
       		    // Call Post for creating the incidence in ACSmi Registry
        		ServiceIncidence performanceincidence = new ServiceIncidence();
    	        performanceincidence.setDetail(availabilityerror);
    	        // tests for the conversion of the timestamp
    	        //SimpleDateFormat sdf = new SimpleDateFormat("YYYY-mm-dd'T'HH:MM:ssZ");
    	       // System.out.println("****** FECHA : " + sdf.format(new Date()));
    	        // locationincidence.setTimestamp (new Date ().getTime());
    	        performanceincidence.setTimestamp("2018-11-24T12:15:00.898Z");
    	        performanceincidence.setServiceid(Long.parseLong(vm.getCsId()));    	       
    	        apiacsmi.createServiceIncidenceUsingPOST(performanceincidence);
    	        System.out.println(" ***** Performance violation to ACSmI ***" + performanceincidence);
    	        // Call VH for creating the violation in the VH
    	        Violation performanceviolation = new Violation();
    	        performanceviolation.setNfr("Performance");
    	        performanceviolation.setServiceId(vm.getCsId());
    	        performanceviolation.timestamp(new Date ().getTime());
    	        // Call to ADAPT VH
    	        // apivh.reportViolation(performanceviolation);
    	        System.out.println(" ***** Performance violation to VH ***" + performanceviolation);
        	}	
        }        			
      }
}



        
        
		

