/****************************************************************
*Copyright (c) 2018 Tecnalia.
*Licensed under MIT License (the "License");
*Permission is hereby granted, free of charge, to any person obtaining a copy 
of this software and associated documentation files (the "Software"), to deal 
in the Software without restriction, including without limitation the rights 
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
copies of the Software, and to permit persons to whom the Software is furnished 
to do so, subject to the following conditions:
*The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.
*
*
*THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE 
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*Contributors (in alphabetical order):
*Alberto Molinuevo 								Tecnalia
*Gorka Benguria                                 Tecnalia
*Iñaki Etxaniz                                  Tecnalia
*Juncal Alonso                                  Tecnalia
*Leire Orue-Echevarria                          Tecnalia
*Maria Jose Lopez                               Tecnalia
*Marisa Escalante                               Tecnalia
*
*Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
*****************************************************************************/
package eu.decideh2020.acsmi.monitoring.server.api.impl;

import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

import org.apache.cxf.jaxrs.client.ClientConfiguration;
import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
import org.apache.cxf.jaxrs.client.WebClient;

import com.fasterxml.jackson.jaxrs.json.JacksonJsonProvider;
import com.maxmind.geoip2.DatabaseReader;
import com.maxmind.geoip2.exception.GeoIp2Exception;
import com.maxmind.geoip2.model.CountryResponse;

import eu.decideh2020.ACSmI.discovery.backend.client.api.ServiceResourceApi;
import eu.decideh2020.ACSmI.discovery.backend.client.model.ServiceNFRInfoVO;

public class NfrLocation {

	String CheckLocation (List <Long> csid, MonitoredVm vm)	{
		// Get the location for the CS
		
		// to open connection with ACSmI client
		ServiceResourceApi api;
		JacksonJsonProvider provider = new JacksonJsonProvider();
		List providers = new ArrayList();
		providers.add(provider);
		String checklocation = null;
		//the integration environment where the ACSmI registry is deployed
		api = JAXRSClientFactory.create("http://85.91.40.245:8087/acsmiservices", ServiceResourceApi.class, providers);
		org.apache.cxf.jaxrs.client.Client client = WebClient.client(api);
	    ClientConfiguration config = WebClient.getConfig(client); 
	    // end of the instructions to connect
	    
	    // Upload the DB with the countries and continents
	    final File database = new File("resources/GeoLite2-Country.mmdb");
	    DatabaseReader dbRead = null;
	      try {
			dbRead = new DatabaseReader.Builder(database).build();
	      } catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	        // Get the NFR for the csid calling ACSmI discovery
	        List<ServiceNFRInfoVO> response = api.getServiceNfrInfoUsingGET(csid);
        	
	        // Start the process to check where is the VM located
        	InetAddress ip;
        	try {
				ip = InetAddress.getByName(vm.getdockerHostPublicIP());
				CountryResponse cresponse = null;
				try {
					cresponse = dbRead.country(ip);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (GeoIp2Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				final String region = cresponse.getContinent().getName();
				final String zone = cresponse.getCountry().getName();
				// Go to all the NFR collected
				for (ServiceNFRInfoVO location : response) {
    			//location.getNfrvalue() con variable region y la variable zone
    			// Zone is not null, both zones are  compared.
        			if (location.getNfrmetric().equals("Zone") && location.getNfrvalue() != null) {
        				if (!location.getNfrvalue().equals(zone)) {
        					return "Error in Zone";
        				}        				     					
        			}
        			else if (location.getNfrmetric().equals("Region")) {
        				if (!location.getNfrvalue().equals(region)) {
        							return "Error in Region";
        						}        						
        					}
 					}
    	} catch (UnknownHostException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
    	}
			return "Location Ok";
			
	}
}
