package eu.decideh2020.adapt.violationhandler.client.api;

import eu.decideh2020.adapt.violationhandler.client.model.Violation;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;
import java.util.Map;
import javax.ws.rs.*;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.MediaType;
import org.apache.cxf.jaxrs.ext.multipart.*;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponses;
import io.swagger.annotations.ApiResponse;
import io.swagger.jaxrs.PATCH;

/**
 * Violation Handler
 *
 * <p>This is the API documentation for Violation Handler service of DECIDE h2020 european project
 *
 */
@Path("/")
@Api(value = "/", description = "")
public interface ViolationsApi  {

    /**
     * Get all the violations that exist in the handler
     *
     * Get all the violations that exist in the handler
     *
     */
    @GET
    @Path("/violation/getAllViolations")
    @Produces({ "application/xml", "application/json" })
    @ApiOperation(value = "Get all the violations that exist in the handler", tags={  })
    @ApiResponses(value = { 
        @ApiResponse(code = 200, message = "successful operation", response = Violation.class, responseContainer = "array") })
    public List<Violation> getAllViolations();

    /**
     * Get violation from id
     *
     * Get violation from id
     *
     */
    @GET
    @Path("/violation/{violationId}")
    @Produces({ "application/xml", "application/json" })
    @ApiOperation(value = "Get violation from id", tags={  })
    @ApiResponses(value = { 
        @ApiResponse(code = 200, message = "successful operation", response = Violation.class),
        @ApiResponse(code = 400, message = "Invalid agent_id value", response = void.class) })
    public Violation getViolationById(@PathParam("violationId") String violationId);

    /**
     * Get all violations for given application description URI
     *
     * Get all violations for given application description URI
     *
     */
    @GET
    @Path("/violation/getViolationsByServiceId")
    @Produces({ "application/xml", "application/json" })
    @ApiOperation(value = "Get all violations for given application description URI", tags={  })
    @ApiResponses(value = { 
        @ApiResponse(code = 200, message = "successful operation", response = Violation.class, responseContainer = "array") })
    public List<Violation> getViolationsByServiceId(@QueryParam("id")Integer id);

    /**
     * Get all violations for given application description URI
     *
     * Get all violations for given application description URI
     *
     */
    @GET
    @Path("/violation/getViolationsByUri")
    @Produces({ "application/xml", "application/json" })
    @ApiOperation(value = "Get all violations for given application description URI", tags={  })
    @ApiResponses(value = { 
        @ApiResponse(code = 200, message = "successful operation", response = Violation.class, responseContainer = "array") })
    public List<Violation> getViolationsByUri(@QueryParam("uri")String uri);

    /**
     * Report a new violation
     *
     * Report a new violation in the system
     *
     */
    @POST
    @Path("/violation")
    @Consumes({ "application/json", "application/json" })
    @Produces({ "application/json", "application/json" })
    @ApiOperation(value = "Report a new violation", tags={  })
    @ApiResponses(value = { 
    @ApiResponse(code = 200, message = "successful operation", response = Violation.class) })
    public Violation reportViolation(Violation violation);
}

