# currently we manually specify draft 06 instead of 07 in schema file $schema
# since gem doesn't support draft 07 and no features introduced in 07 are used in current schema
class ApplicationDescriptionSchema
  FILE_NAME = 'application_description.schema.json'.freeze
  PATH = File.join(Rails.root, 'db', FILE_NAME).freeze

  def self.fix_output(parsed_json)
    errors = vm_errors(parsed_json)
    return parsed_json if errors.blank?
    errors.each do |err|
      prop_data = vm_property_data(err[:property])
      parsed_json['virtualMachines'][err[:arr_index]][err[:property]] = prop_data['minimum'] || prop_data['default']
    end
    raise ACSmIExceptions::InvalidSchema if vm_errors(parsed_json).present?
    parsed_json
  end

  def self.user_friendly_errors(parsed_json)
    errors = validate(parsed_json)
    errors.map do |error|
      error = error.split(', inclusively in schema', 2).first
      error = error.split(' in schema', 2).first
      error.gsub!('did', 'does')
      error[0] = error[0].downcase
      error.gsub!("the property '#/'", 'description file')
      error.gsub!("'#/", "'")
      error.gsub!("property of", 'property')

      error
    end
  end

  def self.vm_data_strict_validate(parsed_json)
    errors = validate(parsed_json)
    return true if errors.select { |err_msg| err_msg =~ /The property '#\/schema\/(\d+)\/(index|csId)'/i }.blank?
    raise ACSmIExceptions::InvalidSchema, errors
  end

  def self.validate(parsed_json)
    schema = JSON.parse File.read(PATH)
    schema['$schema'] = schema['$schema'].gsub('draft-07', 'draft-06')
    JSON::Validator.fully_validate(schema, parsed_json)
  end

  def self.vm_errors(parsed_json)
    errors = validate(parsed_json)
    errors.select! { |err_msg| err_msg =~ /The property '#\/VirtualMachines\/(\d+)\/(\w+)'/i }
    errors.map! { |err| err.gsub("The property '#/virtualMachines/", '') }
    errors.inject([]) do |errors, err|
      vm_arr_index, err = err.split('/', 2)
      vm_invalid_prop, err = err.split("'", 2)
      vm_err = nil
      vm_err = :blank if err.include?('did not have a minimum value of')
      vm_err = :type_mismatch if err.include?('did not match the following type:')
      errors << { arr_index: vm_arr_index.to_i, property: vm_invalid_prop, error: vm_err }

      errors
    end
  end

  def self.vm_property_data(name)
    schema = JSON.parse File.read(PATH)
    schema['definitions']['VirtualMachine']['properties'][name]
  end

  private_class_method :vm_errors, :vm_property_data
end