class Contract < ActiveRecord::Base
  belongs_to :user
  has_many :contracting_resources, dependent: :destroy
  has_many :sessions, through: :contracting_resources

  validates :resource_id, presence: true
  validates :resource_name, presence: true
  validates :user, presence: true

  def self.generate(contracting_resource)
    session = contracting_resource.session
    user = session.user
    user.register_plaform_user if user.platform_email.blank?
    unless PlatformConnector.access_exists?(contracting_resource.virtual_machine.region_id,
                                            user.platform_id)
      PlatformConnector.register_access(contracting_resource.resource_id, user.platform_id)
    end
    contract = Contract.create!(user: user,
                                resource_id: contracting_resource.resource_id,
                                resource_name: contracting_resource.resource_name)
    contracting_resource.update!(contract: contract)
  end

  def store_instance_data(attrs_hash)
    attrs_hash.each do |name, value|
      instance_attributes.build(name: name, value: value)
    end
  end
end