class Service
  attr_reader :id

  def initialize(service_id)
    @id = service_id
  end

  def exists?
    class_attributes.try(&:any?)
  end

  def class_attribute(attr_name)
    class_attributes.try(:[], attr_name)
  end

  def class_attributes
    return nil unless id
    @class_attributes ||= DiscoveryConnector.service_data(id)

    @class_attributes
  end

  def attribute(attr_name, attr_detail = 'value')
    attributes.try(:[], attr_name).try(:[], attr_detail.to_sym)
  end

  def attributes
    return nil unless id
    return @attributes if @attributes

    @attributes = {}

    DiscoveryConnector.service_data(id)['attributes'].each do |attr_hash|
      @attributes[attr_hash['name']] = {value: attr_hash['value'],
                                        human: human_from_hash(attr_hash)}
    end

    @attributes
  end

  def type
    @service_type ||= class_attribute('serviceclassname')
    @service_type
  end

  private

  def human_from_hash(attr_hash)
    return attr_hash['human'] if attr_hash['human']
    "#{attr_hash['value']} #{attr_hash['unitvalue'] || attr_hash['units']}"
  end
end