class InitializeSession
  attr_reader :session, :git_credentials, :app_description, :forced_resources

  def self.call(git_credentials, forced_resources = nil)
    new(git_credentials, forced_resources).perform
  end

  def initialize(git_credentials, forced_resources)
    @session = Session.new(id: SecureRandom.uuid)
    @git_credentials = git_credentials
    @forced_resources = forced_resources
  end

  def perform
    attach_git_access
    download_description_file
    validate_description_by_schema
    attach_contracting_resources
    get_contracting_resources_data
    session.tap(&:save!)
  end

  private

  def attach_git_access
    session.build_git_access(git_credentials)
    session.git_access.ensure_valid
  end

  def download_description_file
    @app_description = ApplicationDescription.new(@session)
    app_description.get
    app_description.validate unless forced_resources
  end

  def validate_description_by_schema
    ApplicationDescriptionSchema.vm_data_strict_validate(app_description.read)
  end

  def attach_contracting_resources
    services_params = []
    if forced_resources.present?
      forced_resources.split(',').each_with_index do |res_id, index|
        services_params << {input_resource_id: res_id.to_i, index: index}
      end
    else
      app_description.read['schema'].each do |service|
        services_params << {input_resource_id: service['csId'], index: service['index']}
      end
    end
    session.contracting_resources.build(services_params)
  end

  def get_contracting_resources_data
    session.contracting_resources.each do |contracting_resource|
      begin
        PrepareResourceAndVM.call(contracting_resource)
      #rescue StandardError
      #  contracting_resource.fail!
      end
    end
  end

end