class ResourceMapper
  @@possible_providers = {
      'amazon' => ['amazon ec2', 'ams3', 'amdb', 'amazon'],
      'cloudsigma' => %w(cloudsigma clsi),
      'arsys' => ['arsys', 'arsy']
  }

  @@optimization_map = {
      'GEPU' => 'generic',
      'COOP' => 'compute',
      'GPUI' => 'gpu',
      'MEOP' => 'memory',
      'SGOP' => 'storage',
      'HPCO' => 'performance',
  }

  @@zone_map = {
      '00AS' => 'asia',
      '00EU' => 'europe',
      '00NA' => 'north america',
      '00SA' => 'south america',
  }

  PROVIDER_NOT_SUPPORTED = 'provider_not_supported'

  def self.map(provider, region, zone, cores, frequency, memory, storage, optimized_for)
    request_params = [
        { field: 'zone', value: match_zone(region) },
        { field: 'cores', operation: '>=', value: cores.to_i },
        { field: 'memory', operation: '>=', value: memory.to_i },
        { field: 'storage', operation: '>=', value: storage.to_i }
    ]

    requested_provider = match_provider(provider)
    request_params << { field: 'provider', value: requested_provider } if requested_provider

    requested_optimization = match_optimization(optimized_for) && requested_provider == 'amazon'
    request_params << { field: 'optimized_for', value: requested_optimization } if requested_optimization

    PlatformConnector.ask_configuration_assistant(request_params)
  end

  class << self

    private

    def match_provider(provider_name)
      provider_name.downcase!
      @@possible_providers.each do |possible_provider, provider_keys|
        provider_keys.each do |provider_key|
          return possible_provider if (provider_name.include?(provider_key) || provider_key.include?(provider_name))
        end
      end
      provider_name
    end

    def match_optimization(optimized_for)
      @@optimization_map[optimized_for]
    end

    def match_zone(region)
      @@zone_map[region]
    end

  end
end