/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.frontend.server.service.dto;

import eu.decideh2020.acsmi.frontend.server.config.Constants;

import eu.decideh2020.acsmi.frontend.server.domain.Authority;
import eu.decideh2020.acsmi.frontend.server.domain.User;

import org.hibernate.validator.constraints.Email;
import org.hibernate.validator.constraints.NotBlank;

import javax.validation.constraints.*;
import java.time.Instant;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * A DTO representing a user, with his authorities.
 */
public class UserDTO {

    private Long id;

    @NotBlank
    @Pattern(regexp = Constants.LOGIN_REGEX)
    @Size(min = 1, max = 50)
    private String login;

    @Size(max = 50)
    private String firstName;

    @Size(max = 50)
    private String lastName;

    @Email
    @Size(min = 5, max = 100)
    private String email;

    @Size(max = 100)
    private String company;

    @Size(min = 1, max = 50)
    private String usersGroup;

    @Size(min = 2, max = 3)
    private String userType;

    @Size(max = 30)
    private String creditCardType;

    @Size(min = 13, max = 16)
    private String creditCard;

    @Size(min = 13, max = 16)
    private String contractReference;

    @Size(min = 8, max = 12)
    private String cifOrVat;

    @Size(max = 256)
    private String imageUrl;

    private boolean activated = false;

    @Size(min = 2, max = 5)
    private String langKey;

    private String createdBy;

    private Instant createdDate;

    private String lastModifiedBy;

    private Instant lastModifiedDate;

    private Set<String> authorities;

    public UserDTO() {
        // Empty constructor needed for Jackson.
    }

    public UserDTO(User user) {
        this(user.getId(), user.getLogin(), user.getFirstName(), user.getLastName(),
            user.getEmail(), user.getCompany(), user.getUsersGroup(), user.getUserType(), user.getCreditCardType(),
            user.getCreditCard(), user.getContractReference(), user.getCifOrVat(), user.getActivated(),
            user.getImageUrl(), user.getLangKey(), user.getCreatedBy(), user.getCreatedDate(),
            user.getLastModifiedBy(), user.getLastModifiedDate(),
            user.getAuthorities().stream().map(Authority::getName)
                .collect(Collectors.toSet()));
    }

    public UserDTO(Long id, String login, String firstName, String lastName,
        String email, String company, String usersGroup, String userType, String creditCardType,
        String creditCard, String contractReference, String cifOrVat, boolean activated,
        String imageUrl, String langKey, String createdBy, Instant createdDate,
        String lastModifiedBy, Instant lastModifiedDate,
        Set<String> authorities) {

        this.id = id;
        this.login = login;
        this.firstName = firstName;
        this.lastName = lastName;
        this.email = email;
        this.company = company;
        this.usersGroup = usersGroup;
        this.userType = userType;
        this.creditCardType = creditCardType;
        this.creditCard = creditCard;
        this.contractReference = contractReference;
        this.cifOrVat = cifOrVat;
        this.activated = activated;
        this.imageUrl = imageUrl;
        this.langKey = langKey;
        this.createdBy = createdBy;
        this.createdDate = createdDate;
        this.lastModifiedBy = lastModifiedBy;
        this.lastModifiedDate = lastModifiedDate;
        this.authorities = authorities;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getLogin() {
        return login;
    }

    public void setLogin(String login) {
        this.login = login;
    }

    public String getFirstName() {
        return firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public String getEmail() {
        return email;
    }

    public String getCompany() {
        return company;
    }

    public String getUsersGroup() {
        return usersGroup;
    }

    public String getUserType() {
        return userType;
    }

    public String getCreditCardType() {
        return creditCardType;
    }

    public String getCreditCard() {
        return creditCard;
    }

    public String getContractReference() {
        return contractReference;
    }

    public String getCifOrVat() {
        return cifOrVat;
    }

    public String getImageUrl() {
        return imageUrl;
    }

    public boolean isActivated() {
        return activated;
    }

    public String getLangKey() {
        return langKey;
    }

    public String getCreatedBy() {
        return createdBy;
    }

    public Instant getCreatedDate() {
        return createdDate;
    }

    public String getLastModifiedBy() {
        return lastModifiedBy;
    }

    public Instant getLastModifiedDate() {
        return lastModifiedDate;
    }

    public void setLastModifiedDate(Instant lastModifiedDate) {
        this.lastModifiedDate = lastModifiedDate;
    }

    public Set<String> getAuthorities() {
        return authorities;
    }

    @Override
    public String toString() {
        return "UserDTO{" +
            "login='" + login + '\'' +
            ", firstName='" + firstName + '\'' +
            ", lastName='" + lastName + '\'' +
            ", email='" + email + '\'' +
            ", company='" + company + '\'' +
            ", usersGroup='" + usersGroup + '\'' +
            ", userType='" + userType + '\'' +
            ", creditCardType='" + creditCardType + '\'' +
            ", creditCard='" + creditCard + '\'' +
            ", contractReference='" + contractReference + '\'' +
            ", cifOrVat='" + cifOrVat + '\'' +
            ", imageUrl='" + imageUrl + '\'' +
            ", activated=" + activated +
            ", langKey='" + langKey + '\'' +
            ", createdBy=" + createdBy +
            ", createdDate=" + createdDate +
            ", lastModifiedBy='" + lastModifiedBy + '\'' +
            ", lastModifiedDate=" + lastModifiedDate +
            ", authorities=" + authorities +
            "}";
    }
}
